##- Nanar <nanardon@mandrake.org>
##-
##- This program is free software; you can redistribute it and/or modify
##- it under the terms of the GNU General Public License as published by
##- the Free Software Foundation; either version 2, or (at your option)
##- any later version.
##-
##- This program is distributed in the hope that it will be useful,
##- but WITHOUT ANY WARRANTY; without even the implied warranty of
##- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##- GNU General Public License for more details.
##-
##- You should have received a copy of the GNU General Public License
##- along with this program; if not, write to the Free Software
##- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Media.pm,v 1.1 2004/08/01 17:52:05 othauvin Exp $

package Hdlist::Media;

use strict;
use Hdlist;

# $options = {
# prefixurl if need, ie real rpms locations
# type => rpms, hdlist, headers
# source => $ (hdlist,dir) or @ (rpms, headers)
sub new {
    my ($class, $options) = @_;
    my $media = {
        headers => [],
        locations => {}, # PKGID => real file, prefix removed
        type => undef, # hdlists, list of file, dir
        prefixurl => undef, #where to find pkg
    };

    if ($options->{type} eq 'rpms') {
        # Adding prefix
        foreach my $rpm (@{$options->{source}}) {
            my $hdr;
            # TODO what to do on error
            $hdr = rpm2header((defined($options->{prefixurl}) ? $options->{prefixurl} :"") .
                    $rpm) and do {
                    $media->{locations}{$hdr->queryformat("PKGID")} = $rpm;
                    push(@{$media->{headers}}, $hdr);
                };
        }
    } elsif ($options->{type} eq 'headers') {
        my $badfiles;
        (undef, $media->{headers}) = Hdlist::parseheaders(@{$options->{source}});
    } elsif ($options->{type} eq 'hdrs') {
        my $badfiles;
        (undef, $media->{headers}) = Hdlist::parsehdrs(@{$options->{source}});
   } elsif ($options->{type} eq 'hdlist') {
        foreach my $h (@{$options->{source}}) {
            my @headers = Hdlist::parsehdlist($h);
            if (@headers) {
                push(@{$media->{headers}}, @headers);
            } else {
            }
        }
    } else {
        die(defined($options->{type}) ?
            "Unknown type of media: $options->{type}" :
            "Type of media not given");
    }
    
    $media->{type} = $options->{type};
    $media->{prefixurl} = $options->{prefixurl};
    bless($media, $class);
}

1;
